/***************************************************************************
                          icewoed.cpp  -  description
                             -------------------
    begin                :    8 19:06:45 EEST 2002
    copyright            : (C) 2002 by Vadim A. Khohlov
    email                : xvadim@teko.kherson.ua
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>

#include <qaccel.h>
#include "icewoed.h"
#include "filesave.xpm"
#include "fileopen.xpm"
#include "icewm.xpm"
#include "exit.xpm"

#include "qcentralwidget.h"

IcewoedApp::IcewoedApp():
icewmCfgDir(getenv("ICEWM_PRIVCFG")),
programName(tr("IceWM's Winoptions editor " VERSION))
{
  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initActions();
  initMenuBar();
  initToolBar();
  initStatusBar();

  viewToolBar->setOn(true);
  viewStatusBar->setOn(true);

  if(icewmCfgDir.isNull())
    icewmCfgDir = QDir::homeDirPath() + "/.icewm";

  centralWidget = new QCentralWidget(this, "central widget", icewmCfgDir);
  setCentralWidget(centralWidget);
  
  if(qApp->argc() > 1)
	optFileName = qApp->argv()[1];
  else
    optFileName = icewmCfgDir + "/winoptions";
  centralWidget->loadOptions(optFileName);
  setCaption(optFileName + " - " + programName);
  fileSave->setEnabled(false);
  fileSaveAs->setEnabled(false);
}/* constructor */

IcewoedApp::~IcewoedApp()
{
}

/** initializes all QActions of the application */
void IcewoedApp::initActions(){

  QPixmap openIcon(fileopen), saveIcon(filesave), iceIcon(icewm),
 		  exitIcon(exit_xpm);

  fileOpen = new QAction(tr("Open File"), openIcon, tr("&Open..."), 0, this);
  fileOpen->setStatusTip(tr("Opens an existing document"));
  fileOpen->setWhatsThis(tr("Open File\n\nOpens an existing document"));
  connect(fileOpen, SIGNAL(activated()), this, SLOT(slotFileOpen()));

  fileSave = new QAction(tr("Save File"), saveIcon,
		  tr("&Save"), QAccel::stringToKey(tr("Ctrl+S")), this);
  fileSave->setStatusTip(tr("Saves the actual document"));
  fileSave->setWhatsThis(tr("Save File.\n\nSaves the actual document"));
  connect(fileSave, SIGNAL(activated()), this, SLOT(slotFileSave()));

  fileSaveAs = new QAction(tr("Save File As"), tr("Save &as..."), 0, this);
  fileSaveAs->setStatusTip(tr("Saves the actual document under a new filename"));
  fileSaveAs->setWhatsThis(tr("Save As\n\nSaves the actual document under a new filename"));
  connect(fileSaveAs, SIGNAL(activated()), this, SLOT(slotFileSave()));

  fileQuit = new QAction(tr("Exit"), exitIcon, tr("E&xit"),
		  QAccel::stringToKey(tr("Ctrl+Q")), this);
  fileQuit->setStatusTip(tr("Quits the application"));
  fileQuit->setWhatsThis(tr("Exit\n\nQuits the application"));
  connect(fileQuit, SIGNAL(activated()), this, SLOT(slotFileQuit()));

  fileRestartIce = new QAction(tr("Restart IceWM"), iceIcon, tr("&Restart IceWM"),
		 				 QAccel::stringToKey(tr("Ctrl+R")), this);
  fileRestartIce->setStatusTip(tr("Restart the IceWM"));
  fileRestartIce->setWhatsThis(tr("Restart\n\nRestarts the IceWM"));
  connect(fileRestartIce, SIGNAL(activated()), this, SLOT(slotRestartIcewm()));

  viewToolBar = new QAction(tr("Toolbar"), tr("Tool&bar"), 0, this, 0, true);
  viewToolBar->setStatusTip(tr("Enables/disables the toolbar"));
  viewToolBar->setWhatsThis(tr("Toolbar\n\nEnables/disables the toolbar"));
  connect(viewToolBar, SIGNAL(toggled(bool)), this, SLOT(slotViewToolBar(bool)));

  viewStatusBar = new QAction(tr("Statusbar"), tr("&Statusbar"), 0, this, 0, true);
  viewStatusBar->setStatusTip(tr("Enables/disables the statusbar"));
  viewStatusBar->setWhatsThis(tr("Statusbar\n\nEnables/disables the statusbar"));
  connect(viewStatusBar, SIGNAL(toggled(bool)), this, SLOT(slotViewStatusBar(bool)));

  helpAboutApp = new QAction(tr("About"), tr("&About..."), 0, this);
  helpAboutApp->setStatusTip(tr("About the application"));
  helpAboutApp->setWhatsThis(tr("About\n\nAbout the application"));
  connect(helpAboutApp, SIGNAL(activated()), this, SLOT(slotHelpAbout()));
}

void IcewoedApp::initMenuBar()
{
  ///////////////////////////////////////////////////////////////////
  // MENUBAR

  ///////////////////////////////////////////////////////////////////
  // menuBar entry fileMenu
  fileMenu=new QPopupMenu();
  fileOpen->addTo(fileMenu);
  fileMenu->insertSeparator();
  fileSave->addTo(fileMenu);
  fileSaveAs->addTo(fileMenu);
  fileMenu->insertSeparator();
  fileRestartIce->addTo(fileMenu);
  fileMenu->insertSeparator();
  fileQuit->addTo(fileMenu);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry viewMenu
  viewMenu=new QPopupMenu();
  viewMenu->setCheckable(true);
  viewToolBar->addTo(viewMenu);
  viewStatusBar->addTo(viewMenu);
  ///////////////////////////////////////////////////////////////////
  // EDIT YOUR APPLICATION SPECIFIC MENUENTRIES HERE

  ///////////////////////////////////////////////////////////////////
  // menuBar entry helpMenu
  helpMenu=new QPopupMenu();
  helpAboutApp->addTo(helpMenu);

  ///////////////////////////////////////////////////////////////////
  // MENUBAR CONFIGURATION
  menuBar()->insertItem(tr("&File"), fileMenu);
  menuBar()->insertItem(tr("&View"), viewMenu);
  menuBar()->insertSeparator();
  menuBar()->insertItem(tr("&Help"), helpMenu);

}

void IcewoedApp::initToolBar()
{
  ///////////////////////////////////////////////////////////////////
  // TOOLBAR
  fileToolbar = new QToolBar(this, "file operations");
  fileOpen->addTo(fileToolbar);
  fileSave->addTo(fileToolbar);
  fileToolbar->addSeparator();
  fileRestartIce->addTo(fileToolbar);
  fileToolbar->addSeparator();
  QWhatsThis::whatsThisButton(fileToolbar);
  fileToolbar->addSeparator();
  fileQuit->addTo(fileToolbar);
}

void IcewoedApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  //STATUSBAR
  statusBar()->message(tr("Ready."), 2000);
}


/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////


void IcewoedApp::slotFileOpen()
{
  statusBar()->message(tr("Opening file..."));
  checkDirty();
  optFileName = QFileDialog::getOpenFileName(icewmCfgDir, 0,this);
  if(!optFileName.isEmpty())
  {
    setCaption(optFileName + " - " + programName);
	fileSave->setEnabled(false);
	fileSaveAs->setEnabled(false);
    QString message=tr("Loaded document: ")+optFileName;
	centralWidget->loadOptions(optFileName);
    statusBar()->message(message, 2000);
  }
  else
  {
    statusBar()->message(tr("Opening aborted"), 2000);
  }
}


void IcewoedApp::slotFileSave()
{
  statusBar()->message(tr("Saving file..."));
  if(!optFileName.isEmpty())
  {
  	centralWidget->saveOptions(optFileName);
    setCaption(optFileName + " - " + programName);
	fileSave->setEnabled(false);
	fileSaveAs->setEnabled(false);
  } /*   if(!optFileName.isEmpty()) */
  else
	slotFileSaveAs();
  statusBar()->message(tr("Ready."));
}

void IcewoedApp::slotFileSaveAs()
{
  statusBar()->message(tr("Saving file under new filename..."));
  optFileName = QFileDialog::getSaveFileName(icewmCfgDir, 0, this);
  if (!optFileName.isEmpty())
  {
	centralWidget->saveOptions(optFileName);
    setCaption(optFileName + " - " + programName);
	fileSave->setEnabled(false);
	fileSaveAs->setEnabled(false);
  } /*   if (!optFileName.isEmpty()) */
  else
    statusBar()->message(tr("Saving aborted"), 2000);
  statusBar()->message(tr("Ready."));
}

void IcewoedApp::slotFileQuit()
{
	checkDirty();
	qApp->quit();
}


void IcewoedApp::slotViewToolBar(bool toggle)
{
  statusBar()->message(tr("Toggle toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off

  if (toggle== false)
  {
    fileToolbar->hide();
  }
  else
  {
    fileToolbar->show();
  };

  statusBar()->message(tr("Ready."));
}

void IcewoedApp::slotViewStatusBar(bool toggle)
{
  statusBar()->message(tr("Toggle statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off

  if (toggle == false)
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  statusBar()->message(tr("Ready."));
}

void IcewoedApp::slotHelpAbout()
{
  QMessageBox::about(this,tr("About..."),
                      tr("IceWM's winoptions editor\nVersion " VERSION "\n(c) 2002-2004 by Vadim A. Khohlov") );
}

void IcewoedApp::slotRestartIcewm()
{
	system("killall -HUP icewm");
} /* void IcewoedApp::slotRestartIcewm() */

/**
* Checks dirty flag of centralwidget and saves menu if necessary.
*/
void
IcewoedApp::checkDirty()
{
	if(centralWidget->isMenuDirty())
		if(QMessageBox::warning(this, tr("Attention!"),
					tr("Content was changed.\nDo you want to save it?"),
					QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes)
			slotFileSave();
} /* IcewoedApp ::checkDirty() */

/**
* Called by centralwidget after changing the content.
* Add '*' into caption.
*/
void
IcewoedApp::contentChanged()
{
	setCaption(optFileName + " [*] - " + programName);
	fileSave->setEnabled(true);
	fileSaveAs->setEnabled(true);
} /* IcemcApp ::contentChanged() */
