/***************************************************************************
                          qcentralwidget.cpp  -  description
                             -------------------
    begin                : Thu Aug 8 2002
    copyright            : (C) 2002 by Vadim A. Khohlov
    email                : xvadim@teko.kherson.ua
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qfile.h>
#include <qfileinfo.h>
#include <qtextstream.h>

#include <qlayout.h>
#include <qlistview.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qpixmap.h>

#include "icewoed.h"
#include "qcentralwidget.h"
#include "qaddnewwindialog.h"
#include "qiconpreview.h"

#include "optnames.h"

QCentralWidget::QCentralWidget(QWidget *parent, const char *name,
			const QString &cfgDir):
QWidget(parent,name),
iconBrowseDialog(0),
iconPath("/usr/share/icons"),
iconLibPath("/usr/local/share/icewm/icons/"),
iconPrivLibPath(cfgDir),
addNewWinDialog(0),
isDirty(false)
{
    QHBoxLayout *topLayout = new QHBoxLayout(this, 0);
    CHECK_PTR(topLayout);

    QVBox *vbOpts = new QVBox(this);
	CHECK_PTR(vbOpts);
    topLayout->addWidget(vbOpts);
	
    winList = new QListView(vbOpts);
    CHECK_PTR(winList);
    winList->addColumn("wm_class");
    winList->addColumn("wm_name");
    winList->addColumn("wm_role");
    winList->addColumn("icon");
//    winList->setSorting(-1);
	
	QHBox *hb = new QHBox(vbOpts);
	CHECK_PTR(hb);
	QPushButton *pb = new QPushButton(tr("&Add..."), hb);
	CHECK_PTR(pb);
	connect(pb, SIGNAL(clicked()), SLOT(slotAdd()));
	pb = new QPushButton(tr("&Edit..."), hb);
	CHECK_PTR(pb);
	connect(pb, SIGNAL(clicked()), SLOT(slotEdit()));

	pb = new QPushButton(tr("&Del"), vbOpts);
	CHECK_PTR(pb);
	connect(pb, SIGNAL(clicked()), SLOT(slotDel()));
		
    vbOpts = new QVBox(this);
    CHECK_PTR(vbOpts);
    topLayout->addWidget(vbOpts);

	hb = new QHBox(vbOpts);
	CHECK_PTR(hb);
	QLabel *lb = new QLabel(tr("&Icon:"), hb);
	edIconName = new QLineEdit(hb);
	lb->setBuddy(edIconName);
	pbBrowseIcon = new QPushButton(tr("&Browse..."), hb);

	hb = new QHBox(vbOpts);
	CHECK_PTR(hb);
	lb  = new QLabel(tr("&WorkSpace:"), hb);
	CHECK_PTR(lb);
	cbWorkSpace = new QComboBox(false, hb);
	CHECK_PTR(cbWorkSpace);
	lb->setBuddy(cbWorkSpace);

	hb = new QHBox(vbOpts);
	CHECK_PTR(hb);
	lb  = new QLabel(tr("&Layer:"), hb);
	CHECK_PTR(lb);
	cbLayer = new QComboBox(false, hb, "cbLayer");
	CHECK_PTR(cbLayer);
	lb->setBuddy(cbLayer);
	cbLayer->insertStrList(layerNames, cLayerNames);
	
	hb = new QHBox(vbOpts);
	CHECK_PTR(hb);
	lb  = new QLabel(tr("&Geometry:"), hb);
	CHECK_PTR(lb);
	edGeometry = new QLineEdit(hb);
	CHECK_PTR(edGeometry);
	lb->setBuddy(edGeometry);
	
	hb = new QHBox(vbOpts);
	CHECK_PTR(hb);
	lb  = new QLabel(tr("&Tray:"), hb);
	CHECK_PTR(lb);
	cbTray = new QComboBox(false, hb, "cbTray");
	CHECK_PTR(cbTray);
	lb->setBuddy(cbTray);
	cbTray->insertStrList(trayOpts, cTrayOpts);

	optListView = new QListView(vbOpts);
	CHECK_PTR(optListView);
	optListView->addColumn(tr("Window's options"));
	optListView->setSorting(-1);

    for(int i = cOptions - 1; i >= 0; i--)
	{
		QCheckListItem *clwOpt = new QCheckListItem(optListView, optsNames[i],
		                         QCheckListItem::CheckBox);
		CHECK_PTR(clwOpt);
	} /* 	for(unsigned i = 0; i < sizeof(optsName) / sizeof(const char*); i++) */

	optList.setAutoDelete(true);

	pb = new QPushButton(tr("&Set"), vbOpts);
	CHECK_PTR(vbOpts);
	connect(pb, SIGNAL(clicked()), SLOT(slotSet()));
	
    connect(winList, SIGNAL(selectionChanged(QListViewItem*)),
                     SLOT(slotWinSelectionChanged(QListViewItem*)));
	connect(pbBrowseIcon, SIGNAL(clicked()), SLOT(slotBrowse()));

	iconPrivLibPath += "/icons/";
}/* constructor */

QCentralWidget::~QCentralWidget()
{
}/* destructor */

/**
 * Loads options from file fileName.
 */
void QCentralWidget::loadOptions(const QString &fileName)
{
    QFile optFile(fileName);
	QString optStr;
	QPixmap pxm;
	QListViewItem *it;
	if(optFile.open(IO_ReadOnly))
	{
		QTextStream optStream(&optFile);
		while((optStr = optStream.readLine()) != QString::null)
		{
			if(!optStr.isEmpty() && optStr[0] != '#')
				parseStr(optStr.stripWhiteSpace());
		} /*while((optStr = optStream.readLine()) != QString::null) */
		optFile.close();
		
		XWinOptInfo *optInfo;
        winList->clear();	
		for(optInfo = optList.first(); optInfo != 0; optInfo = optList.next())
		{
			it = new QListViewItem(winList, optInfo->wm_class(), optInfo->wm_name(),
					optInfo->wm_role());
			if(!optInfo->icon().isEmpty())
			{
				pxm.load(absIconPath(optInfo->icon()));
				if(!pxm.isNull())
					it->setPixmap(3, pxm);
				pxm.resize(0, 0);
			} /*if(!optInfo->icon().isEmpty()) */
		} /* 	for( optInfo = optList.first(); optInfo; optInfo = optList.next()) */
		winList->setCurrentItem(winList->firstChild());

		loadIceOpt(fileName);
		isDirty = false;
	} /* 	if(optFile.open(IO_ReadOnly)) */
} /* loadOptions(const QString &fileName) */

/**
 * Extracts options from string optStr.
 */
void QCentralWidget::parseStr(const QString &optStr)
{
	int point1Pos;
	int point2Pos;
	int semicolonPos;

	point1Pos    = optStr.find('.');
	semicolonPos = optStr.find(':');
	if(point1Pos == -1 || semicolonPos == -1)
	{
        QMessageBox::warning(this, tr("Error"),
                             tr("Wrong string: ") + optStr.data());
		return;
	} /* 	if(point1Pos == -1 || semicolonPos == -1) */

	QString wm_class(optStr.left(point1Pos).stripWhiteSpace());
	QString wm_name;
	QString wm_role;
	
	point2Pos = optStr.find('.', point1Pos + 1);
	if(point2Pos != -1 && point2Pos < semicolonPos)
	{
		wm_name = optStr.mid(point1Pos + 1,
				point2Pos - point1Pos - 1).stripWhiteSpace();
		point1Pos = point2Pos;
	} /* 	if(point2Pos != -1) */
	
	point2Pos = optStr.find('.', point1Pos + 1);
	if(point2Pos != -1 && point2Pos < semicolonPos)
	{
		wm_role = optStr.mid(point1Pos + 1,
				point2Pos - point1Pos - 1).stripWhiteSpace();
		point1Pos = point2Pos;
	} /* 	if(point2Pos != -1) */
	QString optName(optStr.mid(point1Pos + 1,
			   	semicolonPos - point1Pos - 1).stripWhiteSpace());
	QString optValue(optStr.mid(semicolonPos + 1).stripWhiteSpace());
	
	/* Finding corresponding winoption's object */
	XWinOptInfo *optInfo;
	
	for(optInfo = optList.first(); optInfo != 0; optInfo = optList.next())
	{
		if(optInfo->wm_class() == wm_class && optInfo->wm_name() == wm_name &&
				optInfo->wm_role() == wm_role)
			break;
	} /* 	for(optInfo = optList.first(); optInfo != 0; optInfo = optList.next()) */
	
	if(!optInfo)
	{
		/* It's first appearance of this wm_class.
		 * We must add new item into list.
		 */
		optList.append(new XWinOptInfo(wm_class, wm_name, wm_role));
		optInfo = optList.last();
	} /* 	if(!optInfo) */
	
	/* Setting options 
	 */
	bool ok;
	int  i;
	QString ovalue(optValue);
	QString oname(optName);
	if(oname == "icon")
	{
		optInfo->setIcon(optValue);
		return;
	} /* 	if(oname == "icon") */
	if(oname == "workspace")
	{
		optInfo->setWorkSpace(optValue.toInt(&ok));
		return;
	} /* 	if(oname == "workspace") */
	if(oname == "layer")
	{
		for(i = cLayerNames - 1; i >=0; i--)
			if( ovalue == QString(layerNames[i]))
				break;
		
		optInfo->setLayer(i < 0 ? 0 : i);
		return;
	} /* 	if(optName == "layer") */
	if(oname == "geometry")
	{
		optInfo->setGeometry(optValue);
		return; 
	} /* 	if(oname == "geometry") */
	if(oname == "tray")
	{
		for(i = cTrayOpts - 1; i >=0; i--)
			if( ovalue == QString(trayOpts[i]))
				break;
		
		optInfo->setTray(i < 0 ? 0 : i);
		return;
	} /* 	if(optName == "layer") */

	for(i = cOptions - 1; i >= 0; i--)
		if(oname == QString(optsNames[i]))
			break;
	if(i >= 0)
		optInfo->setOption(i, optValue.toInt(&ok));
	else
        QMessageBox::warning(this, tr("Error"),
                             tr("Unknown option: ") + optName.data());
} /*   void QCentralWidget::parseStr(const QString &optStr) */

/**
 * This is slot for list of windows.
 */
void QCentralWidget::slotWinSelectionChanged(QListViewItem *it)
{
	QString wm_class(it->text(0));
	QString wm_name(it->text(1));
	QString wm_role(it->text(2));
	XWinOptInfo *optInfo;
	
	for(optInfo = optList.first(); optInfo != 0; optInfo = optList.next())
	{
		if(optInfo->wm_class() == wm_class && optInfo->wm_name() == wm_name &&
				optInfo->wm_role() == wm_role)
			break;
	} /* 	for( optInfo = optList.first(); optInfo; optInfo = optList.next()) */
	if(!optInfo)
        QMessageBox::warning(this, tr("Error"),
                             tr("Undefined item: ") + wm_class.data());
	else
	{
		edIconName->setText(optInfo->icon());
		cbWorkSpace->setCurrentItem(optInfo->workSpace() + 1);
		cbLayer->setCurrentItem(optInfo->layer());
		edGeometry->setText(optInfo->geometry());
		cbTray->setCurrentItem(optInfo->tray());
		
		XWinOptInfo *optInfo;
	
		for(optInfo = optList.first(); optInfo != 0; optInfo = optList.next())
		{
			if(optInfo->wm_class() == wm_class && optInfo->wm_name() == wm_name &&
					optInfo->wm_role() == wm_role)
				break;
		} /* 	for( optInfo = optList.first(); optInfo; optInfo = optList.next()) */
		if(!optInfo)
		{
			debug("Unknown window %s", wm_class.data());
			return;
		} /* 		if(!optInfo) */
		
		QListViewItem *it = optListView->firstChild();
		int i = 0;
		
		while(it)
		{
			((QCheckListItem*)it)->setOn(optInfo->option(i++));
			it = it->nextSibling();
		} /* 		while(it) */
	} /* 	else if(!optInfo)*/
} /* void QCentralWidget::slotWinSelectionChanged(QListViewItem *it) */

/** No descriptions */
void QCentralWidget::saveOptions(const QString &fileName)
{
    //save old copy of file
    QString bakFileName(fileName);
    bakFileName += ".bak";
	QDir d;
    //std::rename( QFile::encodeName(fileName), QFile::encodeName(bakFileName));
    d.rename(fileName, bakFileName);

    QFile optFile(fileName);
    if(!optFile.open(IO_WriteOnly))
    {
        QMessageBox::warning(this, tr("Error"),
                             tr("Can't open ") + fileName);
        return;
    }/* if(!optFile->open()) */
	
	QTextStream optStream(&optFile);
	XWinOptInfo *optInfo;
	/**
	 * Saving options. We don't save default options.
	 */
	for(optInfo = optList.first(); optInfo != 0; optInfo = optList.next())
	{
		int i;
		QString wm_classname(optInfo->wm_class());
		if(!optInfo->wm_name().isEmpty())
		{
			if(!wm_classname.isEmpty())
				wm_classname += '.';
			wm_classname += optInfo->wm_name();
		} /* 		if(!optInfo->wm_name().isEmpty()) */
		if(!optInfo->wm_role().isEmpty())
		{
			if(!wm_classname.isEmpty())
				wm_classname += '.';
			wm_classname += optInfo->wm_role();
		} /* 		if(!optInfo->wm_name().isEmpty()) */

		if(!optInfo->icon().isEmpty())
			optStream << wm_classname << ".icon: " << optInfo->icon() << "\n";
		
		if(optInfo->workSpace() != -1)
			optStream << wm_classname << ".workspace: " << optInfo->workSpace() << "\n";
		
		if(optInfo->layer() != 2)
			optStream << wm_classname << ".layer: " << layerNames[optInfo->layer()] << "\n";
		
		if(!optInfo->geometry().isEmpty())
			optStream << wm_classname << ".geometry: " << optInfo->geometry() << "\n";

		if(optInfo->tray())
			optStream << wm_classname << ".tray: " << trayOpts[optInfo->tray()] << "\n";

		for(i = 0; i < first1; i++)
			if(optInfo->option(i))
				optStream << wm_classname << "." << optsNames[i] << ": 1\n";
		
		for( ;i <= last1; i++)
			if(!optInfo->option(i))
				optStream << wm_classname << "." << optsNames[i] << ": 0\n";

		for( ;i < cOptions; i++)
			if(optInfo->option(i))
				optStream << wm_classname << "." << optsNames[i] << ": 1\n";
		optStream << "\n";
	} /* 	for(optInfo = optList.first(); optInfo != 0; optInfo = optList.next()) */
	optFile.close();
	isDirty = false;
}/* saveOptions(const QString &fileName) */

/**
 * Sets options for selected window.
 */
void QCentralWidget::slotSet()
{
	QListViewItem *it = winList->currentItem();
	QString wm_class(it->text(0));
	QString wm_name(it->text(1));
	QString wm_role(it->text(2));
	XWinOptInfo *optInfo;
	
	for(optInfo = optList.first(); optInfo != 0; optInfo = optList.next())
	{
		if(optInfo->wm_class() == wm_class && optInfo->wm_name() == wm_name &&
				optInfo->wm_role() == wm_role)
			break;
	} /* 	for( optInfo = optList.first(); optInfo; optInfo = optList.next()) */
	if(!optInfo)
	{
        QMessageBox::warning(this, tr("Error"),
                             tr("Undefined item: ") + wm_class.data());
		return;
	} /* 	if(!optInfo) */
	
	optInfo->setIcon(edIconName->text());
	optInfo->setWorkSpace(cbWorkSpace->currentItem() - 1);
	optInfo->setLayer(cbLayer->currentItem());
	optInfo->setGeometry(edGeometry->text());
	optInfo->setTray(cbTray->currentItem());
	
	//QPixmap pxm(absIconPath(optInfo->icon()));
	//if(!pxm.isNull())
		it->setPixmap(3, QPixmap(absIconPath(optInfo->icon())));
	
	it = optListView->firstChild();
	int i = 0;
	while(it)
	{
		optInfo->setOption(i++, ((QCheckListItem*)it)->isOn());
		it = it->nextSibling();
	} /* 		while(it) */
	if(!isDirty)
	{
		isDirty = true;
		((IcewoedApp*)parentWidget())->contentChanged();
	} /* 	if(!isDirty) */
} /* void QCentralWidget::slotSet() */

/** 
 * Adds new window into list.
 */
void QCentralWidget::slotAdd()
{
	if(!addNewWinDialog)
		addNewWinDialog = new QAddNewWinDialog(this);
	if(!addNewWinDialog->exec())
		return;
	
	QString wm_class(addNewWinDialog->wm_class());
	QString wm_name(addNewWinDialog->wm_name());
	QString wm_role(addNewWinDialog->wm_role());
	
	optList.append(new XWinOptInfo(wm_class, wm_name, wm_role));
	new QListViewItem(winList, wm_class, wm_name, wm_role);
	winList->setCurrentItem(winList->firstChild());
} /* void QCentralWidget::slotAdd() */

/** 
 * Deletes selected window from list.
 */
void QCentralWidget::slotDel()
{
	QListViewItem *it = winList->currentItem();
	QString wm_class(it->text(0));
	QString wm_name(it->text(1));
	QString wm_role(it->text(2));
	XWinOptInfo *optInfo;
	
	for(optInfo = optList.first(); optInfo != 0; optInfo = optList.next())
	{
		if(optInfo->wm_class() == wm_class && optInfo->wm_name() == wm_name &&
				optInfo->wm_role() == wm_role)
			break;
	} /* 	for( optInfo = optList.first(); optInfo; optInfo = optList.next()) */
	if(!optInfo)
	{
        QMessageBox::warning(this, tr("Error"),
                             tr("Undefined item: ") + wm_class.data());
		return;
	} /* 	if(!optInfo) */

	optList.removeRef(optInfo);
   	if(!isDirty)
	{
		isDirty = true;
		((IcewoedApp*)parentWidget())->contentChanged();
	} /* 	if(!isDirty) */

	delete it;
} /* void QCentralWidget::slotDel() */

/**
 * Opens dialog for browsing icons.
 */
void QCentralWidget::slotBrowse()
{
    QString fileName;

	if(!iconBrowseDialog)
	{
		QIconPreview *p = new QIconPreview();
    	iconBrowseDialog = new QFileDialog(iconPath,QString::null,this,
                                        "newIcon",true);
    	QString filterTypes(tr("Images (*.xpm *.png *.jpg *.gif *.bmp)"
                         ";;All files (*)"));
    	iconBrowseDialog->setFilters(filterTypes);
    	iconBrowseDialog->setMode(QFileDialog::ExistingFile);
    	iconBrowseDialog->setCaption(tr("Please Select Icon"));
    	iconBrowseDialog->setContentsPreviewEnabled(true);
    	iconBrowseDialog->setContentsPreview(p,p);
    	iconBrowseDialog->setPreviewMode(QFileDialog::Contents);
	} /* 	if(!iconBrowseDialog) */

    if(!iconBrowseDialog->exec())
        return;
    fileName = iconBrowseDialog->selectedFile();

	if (!fileName.isEmpty())
	{
		QFileInfo fi(fileName);
		QString iPath(fi.dirPath(true));
		if(iceIconPath.find(iPath) != -1)
			edIconName->setText(fi.fileName());
		else
		{	//may be it's icon in lib path
			iPath += '/';
			int pos;
			pos = fi.fileName().findRev("_16x16.xpm");
			if(pos == -1)
				pos = fi.fileName().findRev("_32x32.xpm");
			if((iPath == iconLibPath || iPath == iconPrivLibPath) && pos != -1)
				edIconName->setText(fi.fileName().remove(pos,10));
			else
				edIconName->setText(fileName);
		} /* else if(iconPath.find(iPath) != -1) */
	} /* 	if (!fileName.isEmpty()) */
} /*   void slotBrowse() */

/**
 * Loads IceWM options: IconPath and names of workspaces.
 */
void QCentralWidget::loadIceOpt(const QString &fileName)
{
	int i, j;
	QString s(fileName);
	QString str;
	
	i = s.findRev('/');
	s.replace(i + 1, 100, "preferences");

	QFile prefFile(s);

    if(prefFile.open(IO_ReadOnly))
    {
    	QTextStream prefT(&prefFile);
        while ((s = prefT.readLine()) != QString::null)
        {
			if(s[0] == '#')
				continue;
        	str = s.stripWhiteSpace();
            if(str.find("IconPath") != -1)
            {
				i = str.find('\"');
				str.remove(0, i + 1);
				i = str.findRev('\"');
				str.remove(i, 100);
				iceIconPath = str;
				iconPathsLst = QStringList::split(":", iceIconPath);
			} /* if(bgStr.find("IconPath") != -1) */
			
			if(str.find("WorkspaceNames") != -1)
			{
				i = str.find('\"');
				str.remove(0, i);
				i = 0;
				cbWorkSpace->insertItem(tr("Any"));
				while(1)
				{
					j = str.find(',', i);
					if(j == -1)
						break;
					cbWorkSpace->insertItem(str.mid(i, j - i).stripWhiteSpace());
					i = j + 1;
				} /* while(1) */	 
				cbWorkSpace->insertItem(str.mid(i).stripWhiteSpace());
			} /* if(str.find("WorkspaceNames") != -1) */
			
		} /* while ((s = prefT.readLine()) != QString::null) */
		prefFile.close();
    }/* if(prefFile.open(IO_ReadOnly)) */

} /*   void loadIceOpt(const QString &fileName) */


/**
* Edits wm_class and wm_name for selected window.
*/
void
QCentralWidget ::slotEdit()
{
	QListViewItem *it = winList->currentItem();
	QString wm_class(it->text(0));
	QString wm_name(it->text(1));
	QString wm_role(it->text(2));
	XWinOptInfo *optInfo;
	
	for(optInfo = optList.first(); optInfo != 0; optInfo = optList.next())
	{
		if(optInfo->wm_class() == wm_class && optInfo->wm_name() == wm_name &&
				optInfo->wm_role() == wm_role)
			break;
	} /* 	for( optInfo = optList.first(); optInfo; optInfo = optList.next()) */
	if(!optInfo)
	{
        QMessageBox::warning(this, tr("Error"),
                             tr("Undefined item: ") + wm_class.data());
		return;
	} /* 	if(!optInfo) */
	
	if(!addNewWinDialog)
		addNewWinDialog = new QAddNewWinDialog(this);

	addNewWinDialog->setWM_opt(it->text(0), it->text(1), it->text(2));
	if(!addNewWinDialog->exec())
		return;
	
	it->setText(0, addNewWinDialog->wm_class());
	it->setText(1, addNewWinDialog->wm_name());
	it->setText(2, addNewWinDialog->wm_role());

	optInfo->setNewNames(addNewWinDialog->wm_class(),
			addNewWinDialog->wm_name(), addNewWinDialog->wm_role());
} /* QCentralWidget ::slotEdit() */


/**
* Returns absolute filename of icon.
*/
QString
QCentralWidget::absIconPath(const QString &icon)
{
	if(icon[0] != '/')
	{
		QString fullName(iconPrivLibPath + icon + "_16x16.xpm");
		if(QFile::exists(fullName))
			return fullName;
		
		fullName = iconLibPath + icon + "_16x16.xpm";
		if(QFile::exists(fullName))
			return fullName;
		
		for( QStringList::Iterator it = iconPathsLst.begin(); 
						it != iconPathsLst.end(); ++it)
		{
			fullName = *it + '/' + icon;
			if(QFile::exists(fullName))
				return fullName;
		} /*for( it!=iconPathsLst.end(); ++it) 	*/
	} /* 	if(icon[0] != '/') */
	return icon;
} /* QCentralWidget ::absIconPath(const QString &icon) */
