/***************************************************************************
                          qcentralwidget.h  -  description
                             -------------------
    begin                : Thu Aug 8 2002
    copyright            : (C) 2002 by Vadim A. Khohlov
    email                : xvadim@teko.kherson.ua
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef QCENTRALWIDGET_H
#define QCENTRALWIDGET_H

#include <qwidget.h>
#include <qlist.h>
#include <qstringlist.h>

#include "xwinoptinfo.h"

class QListView;
class QListViewItem;
class QPushButton;
class QLineEdit;
class QComboBox;
class QFileDialog;

class QAddNewWinDialog;

/**
 * Central widget with all controls.
 * @author Vadim A. Khohlov
 */

class QCentralWidget : public QWidget  {
   Q_OBJECT
public: 
	QCentralWidget(QWidget *parent, const char *name,
			const QString &cfgDir);
	~QCentralWidget();
  /** 
   * Loads options from file fileName.
   */
  void loadOptions(const QString &fileName);
  /** No descriptions */
  void saveOptions(const QString &fileName);
  bool isMenuDirty();
private: // Private attributes
  /**
   * This is a list of windows' classes and names.
   */
  QListView *winList;
  /**
   * The list with the checkboxes for options.
   */
  QListView *optListView;

  QLineEdit   *edIconName;
  QPushButton *pbBrowseIcon;

  //QLineEdit   *edWorkSpace;
  QComboBox   *cbWorkSpace;
  QComboBox   *cbLayer;
  QLineEdit   *edGeometry;
  QComboBox   *cbTray;

  QList<XWinOptInfo> optList;
  /**  */
  QFileDialog *iconBrowseDialog;
  /**  */
  QString iconPath;
  QString iceIconPath;
  /**
   * The global IceWM icons' path (/usr/local/share/icewm/icons).
   */
  QString iconLibPath;
  /**
   * The private icons' library path (~/.icewm/icons).
   */
  QString	  iconPrivLibPath;
  /**
   * The list of icons' dirs from prefernces.
   */
  QStringList iconPathsLst;

  /**  */
  QAddNewWinDialog *addNewWinDialog;
  
  /**
   * Extracts options from string optStr.
   */
  void parseStr(const QString &optStr);

  /**
   * Loads IceWM options: IconPath and names of workspaces.
   */
  void loadIceOpt(const QString &fileName);
  /**
   * Returns absolute filename of icon.
   */
  QString absIconPath(const QString &icon);
   
  /**
   * a dirty flag - true if menu changed.
   */
  bool isDirty;
  
private slots: // Private slots
  /** 
   * This is slot for list of windows.
   */
  void slotWinSelectionChanged(QListViewItem *it);
  /** 
   * Deletes selected window from list.
   */
  void slotDel();
  /** 
   * Adds new window into list.
   */
  void slotAdd();
  /**
   * Edits wm_class and wm_name for selected window.
   */
  void slotEdit();
  /** 
   * Sets options for selected window.
   */
  void slotSet();
  /**
   * Opens dialog for browsing icons.
   */
  void slotBrowse();
}; /* QCentralWidget */

inline
bool QCentralWidget::isMenuDirty()
{
    return isDirty;    
}/* isMenuDirty() */

#endif

