/***************************************************************************
                          xwinoptinfo.h  -  description
                             -------------------
    begin                : Thu Aug 8 2002
    copyright            : (C) 2002 by Vadim A. Khohlov
    email                : xvadim@teko.kherson.ua
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef XWINOPTINFO_H
#define XWINOPTINFO_H

#include <stdlib.h>

#include <qstring.h>

#include "optnames.h"

/**
 * Class which contains all information
 * about window's options.
 * @author Vadim A. Khohlov
 */

class XWinOptInfo {
public:
    /**
     * Creates a object with default options for window with
     * WM_CLASS aClass and WM_NAME aName.
     */
	XWinOptInfo(const QString &aClass, const QString aName = QString::null,
			const QString &aRole = QString::null);
	~XWinOptInfo();
	/**
	  enum TypeOptions { 
			AllWorkSpace = 0, IgnoreWinList, IgnoreTaskBar, IgnoreQuickSwitch,
			FullKeys,
			FMove, FResize, FClose, FMinimize, FHide, FRollup,
			DTitleBar, DSysMenu, DBorder, DResize, DClose, DMinimize, DMaximize,
			NoFocusOnAppRaise, IgnoreNoFocusHint, DoNotCover
	     };
	*/
	
private: // Private attributes
    /**
     * WM_CLASS and WM_NAME and WM_WINDOW_ROLE of window.
     */
    QString oWm_class;
    QString oWm_name;
    QString oWm_role;

    /**
     * window's options.
     */
    QString oIconName;
	/**
	 * The number of workSpace. if -1 this options not used.
	 */
    int     oWorkSpace;
    int     oLayer;
    QString oGeometry;
    int     oTray;

    int 	oOptions[COPTIONS];
	
public:
	QString wm_class()	{ return oWm_class; }
	QString wm_name()	{ return oWm_name; }
	QString wm_role()	{ return oWm_role; }
	
	QString icon()		{ return oIconName;	} 	
	int workSpace()		{ return oWorkSpace; }
	int layer()			{ return oLayer; }
	QString geometry()	{ return oGeometry; }
	int tray()			{ return oTray; }
	int option(int t);

	void setIcon(const QString &iconName)	{ oIconName = iconName; }
	void setWorkSpace(int num)				{ oWorkSpace = num; }
	void setLayer(int num)					{ oLayer = num; }
	void setGeometry(const QString &geom)	{ oGeometry = geom; }
	void setTray(int num)					{ oTray = num; }
	void setOption(int t, int value);
	void setNewNames(const QString &wclass, const QString &wname,
			const QString &wrole)
	{
		oWm_class = wclass; oWm_name = wname; oWm_role = wrole;
	} 

}; /*  XWinOptInfo */

inline
int  XWinOptInfo::option(int t)
{
	ASSERT(t >= 0 && t <= cOptions);
	return oOptions[t];
} /* int  XWinOptInfo::option(TypeOptions t) */

inline
void  XWinOptInfo::setOption(int t, int value)
{
	ASSERT(t >= 0 && t <= cOptions && (value == 0 || value == 1));
	oOptions[t] = value;
} /* void  XWinOptInfo::setOption(TypeOptions t, int value) */
#endif

